import os
import sys
import logging
import h5py

from nexgen.geometry_phil import scope
from nexgen.writer import NexusWriter

logger = logging.getLogger("NXSWriter")


def main():
    args = sys.argv[1:]
    clai = scope.command_line_argument_interpreter()
    working = scope.fetch(clai.process_and_fetch(args))
    params = working.extract()

    logdir = os.path.dirname(params.output.master_file_name)
    logging.basicConfig(
        filename=os.path.join(logdir, "NexusWriter.log"),
        format="%(message)s",
        level="DEBUG",
    )

    ext = os.path.splitext(params.output.master_file_name)[1]
    print(ext)
    # assert (
    #    (ext == ".nxs") or (ext == ".h5")
    # ), "Wrong file extension, please pass a .h5 or .nxs file."
    # assert (
    #    os.path.splitext(params.output.data_file_template) == ".h5"
    # ), "Wrong file extension, please pass a .h5 or .nxs file."

    logger.info("Nexus file name %s" % params.output.master_file_name)
    cf = params.input.coordinate_frame
    logger.info("Coordinate system: %s" % cf)
    if cf == "imgcif":
        logger.warning("Coordinates will be converted to mcstas.")

    print("Detector information:")
    d = params.input.detector
    print(d.description)
    print(f"Sensor made of {d.sensor_material} x {d.sensor_thickness}mm")
    print(f"Trusted pixels > {d.underload} and < {d.overload}")
    print(f"Image is a {d.image_size} array of {d.pixel_size} mm pixels")

    logger.info("Data type: %s" % d.mode)
    logger.info("")

    logger.info("Detector information: %s" % d.description)
    logger.info(f"Sensor made of {d.sensor_material} x {d.sensor_thickness}mm")
    if d.mode == "images":
        logger.info(f"Trusted pixels > {d.underload} and < {d.overload}")
    logger.info(f"Image is a {d.image_size} array of {d.pixel_size} mm pixels")

    logging.info("Axes:")
    for ax, dep, t, u, j in zip(
        d.axes, d.depends, d.types, d.units, range(len(d.axes))
    ):
        vector = d.vectors[3 * j : 3 * j + 3]
        logger.info("%s %s %s %s %s %s" % (ax, dep, t, u, vector, d.starts[j]))

    if d.flatfield is None:
        logger.info("No flatfield applied")
    else:
        logger.info(f"Flatfield correction data lives here {d.flatfield}")

    if d.pixel_mask is None:
        logger.info("No bad pixel mask for this detector")
    else:
        logger.info(f"Bad pixel mask lives here {d.pixel_mask}")

    logger.warning(f"module_offset field setting: {d.module_offset}")
    logger.info("")

    axes = d.axes
    axis_vectors = d.vectors

    # one day other units could be legal.. but check things match up
    for tu in zip(d.types, d.units):
        assert tu in (("translation", "mm"), ("rotation", "deg"))

    assert len(axis_vectors) == 3 * len(axes)

    for j in range(len(axes)):
        vector = axis_vectors[3 * j : 3 * j + 3]
        print(f"Detector axis: {axes[j]} => {vector}")

    print("Goniometer information:")

    g = params.input.goniometer
    axes = g.axes
    axis_vectors = g.vectors

    for tu in zip(g.types, g.units):
        assert tu in (("translation", "mm"), ("rotation", "deg"))

    assert len(axis_vectors) == 3 * len(axes)

    for j in reversed(range(len(axes))):
        vector = axis_vectors[3 * j : 3 * j + 3]
        print(
            f"Goniometer axis: {axes[j]} => {vector} ({g.types[j]}) on {g.depends[j]}"
        )

    logger.info("Goniometer information")
    for ax, dep, t, u, j in zip(
        g.axes, g.depends, g.types, g.units, range(len(g.axes))
    ):
        vector = g.vectors[3 * j : 3 * j + 3]
        offset = g.offsets[3 * j : 3 * j + 3]
        logger.info(
            "%s %s %s %s %s %s %s %s %s"
            % (ax, dep, t, u, vector, offset, g.starts[j], g.ends[j], g.increments[j])
        )

    logger.info("")

    with h5py.File(params.output.master_file_name, "w") as nxsfile:
        NexusWriter(nxsfile, params).write()

    logger.info("==" * 50)


if __name__ == "__main__":
    main()



"""
Command line tool to copy metadata from LATRD Tristan data collection to a new nexus file.
"""

import os
import sys

import freephil

# from ..nxs_copy import CopyTristanNexus

from nexgen.nxs_copy import CopyTristanNexus

tristan_scope = freephil.parse(
    """
    input {
      data_file = None
        .type = path
        .help = "HDF5 file containing the newly binned images."
      tristan_nexus_file = None
        .type = path
        .help = "NeXus file with experiment metadata to be copied."
      experiment_type = *single multiple pump-probe
        .type = choice
        .help = "Define the type of experiment that has been run."
      oscillation = None
        .type = float
        .help = "Image oscillation angle (degrees)"
      nbins = None
        .type = int
        .help = Number of images that have been binned.
      mode = *static rotation
        .type = choice
        .help = "For pump-probe experiments, define whether it's a rotation or not."
    }
    """
)


def main():
    args = sys.argv[1:]
    clai = tristan_scope.command_line_argument_interpreter()
    working = tristan_scope.fetch(clai.process_and_fetch(args))
    params = working.extract()

    print(
        "Create a NeXus file to go with images binned from LATRD Tristan detector event data."
    )
    wdir = os.path.dirname(params.input.data_file)
    data_file_name = os.path.basename(params.input.data_file)
    print(f"Working directory: {wdir}")
    print(f"Image data file name: {data_file_name}")
    print(f"Original Tristan NeXus file: {params.input.tristan_nexus_file}")

    print(f"Experiment type: {params.input.experiment_type}")
    if params.input.experiment_type == "single":
        print("A single images binned from all the events.")
        CopyTristanNexus.single_image_nexus(
            params.input.data_file, params.input.tristan_nexus_file
        )
    elif params.input.experiment_type == "multiple":
        print("Rotation dataset.")
        if params.input.oscillation and params.input.nbins:
            raise ValueError("oscillation and nbins are mutually exclusive.")
        # print("Oscillation: %.2f deg/s" % params.input.oscillation)
        CopyTristanNexus.multiple_images_nexus(
            params.input.data_file,
            params.input.tristan_nexus_file,
            osc=params.input.oscillation,
            nbins=params.input.nbins,
        )
    elif params.input.experiment_type == "pump-probe":
        print("Pump-probe experiment.")
        print(f"Mode: {params.input.mode}")
        CopyTristanNexus.pump_probe_nexus(
            params.input.data_file,
            params.input.tristan_nexus_file,
            mode=params.input.mode,
        )
    else:
        sys.exit(f"Please pass a valid experiment type.")
    print("All done!")


if __name__ == "__main__":
    main()



"""
Command line tool to copy metadata from existing NeXus files.
"""

import os
import sys

import freephil

from nexgen.nxs_copy import CopyNexus

# from ..nxs_copy import CopyNexus

copy_scope = freephil.parse(
    """
    input {
      data_file_name = None
        .type = path
        .help = "HDF5 data file."
      original_nexus_file = None
        .type = path
        .help = "NeXus file with experiment metadata to be copied."
      mode = *images pseudo-events
        .type = choice
        .help = "Type od data in HDF5 file."
      simple_copy = False
        .type = bool
        .help = "If True, the full NeXus tree is copied, including NXdata."
    }
    """
)


def main():
    args = sys.argv[1:]
    clai = copy_scope.command_line_argument_interpreter()
    working = copy_scope.fetch(clai.process_and_fetch(args))
    params = working.extract()

    print("Copy metadata from one NeXus file to another.")
    data_file = os.path.basename(params.input.data_file_name)
    nexus_file = os.path.basename(params.input.original_nexus_file)
    print(f"Image data file name: {data_file}")
    print(f"Original Tristan NeXus file: {nexus_file}")

    print(f"Data mode: {params.input.mode}")
    if params.input.mode == "images":
        if params.input.simple_copy is True:
            CopyNexus.images_nexus(
                params.input.data_file_name,
                params.input.original_nexus_file,
                simple_copy=params.input.simple_copy,
            )
        else:
            CopyNexus.images_nexus(
                params.input.data_file_name, params.input.original_nexus_file
            )
    elif params.input.mode == "pseudo-events":
        CopyNexus.pseudo_events_nexus(
            params.input.data_file_name, params.input.original_nexus_file
        )
    else:
        sys.exit(f"Please pass a valid data mode.")
    print("All done!")


if __name__ == "__main__":
    main()
